@@ -26,7 +26,6 @@
 #include <e2k-properties.h>
 #include <e2k-propnames.h>
 #include "libecal/e-cal-component.h"
-#include "e-util/e-config-listener.h"
 #include "e2k-cal-utils.h"
 #include <e2k-context.h>
 #include <exchange-account.h>
@@ -46,7 +45,7 @@ typedef struct {
 /* Private part of the ECalBackendExchangeTasks structure */
 struct _ECalBackendExchangeTasksPrivate {
 	/* URI where the task data is stored */
-	char *uri;
+	gchar *uri;
 
 	/* Top level VTODO component */
 	icalcomponent *icalcomp;
@@ -64,18 +63,18 @@ struct _ECalBackendExchangeTasksPrivate {
 
 	gboolean is_loaded;
 
-	int dummy;
+	gint dummy;
 };
 
 #define PARENT_TYPE E_TYPE_CAL_BACKEND_EXCHANGE
 static ECalBackendExchange *parent_class = NULL;
 
 static void
-get_from (ECalBackendSync *backend, ECalComponent *comp, char **from_name, char **from_addr)
+get_from (ECalBackendSync *backend, ECalComponent *comp, gchar **from_name, gchar **from_addr)
 {
 	if (!g_ascii_strcasecmp(e_cal_backend_exchange_get_owner_email (backend), exchange_account_get_email_id (E_CAL_BACKEND_EXCHANGE (backend)->account)))
 		e_cal_backend_exchange_get_from (backend, comp, from_name, from_addr);
-	else 
+	else
 		e_cal_backend_exchange_get_sender (backend, comp, from_name, from_addr);
 
 #if 0
@@ -99,7 +98,7 @@ get_from (ECalBackendSync *backend, ECalComponent *comp, char **from_name, char
 static void
 set_uid (E2kProperties *props, ECalComponent *comp)
 {
-        const char *uid;
+        const gchar *uid;
 
 	e_cal_component_get_uid (E_CAL_COMPONENT (comp), &uid);
         e2k_properties_set_string (props, E2K_PR_CALENDAR_UID, g_strdup (uid));
@@ -121,7 +120,7 @@ set_summary (E2kProperties *props, ECalComponent *comp)
 static void
 set_priority (E2kProperties *props, ECalComponent *comp)
 {
-        int *priority, value = 0;
+        gint *priority, value = 0;
 
 	e_cal_component_get_priority (E_CAL_COMPONENT (comp), &priority);
         if (priority) {
@@ -142,7 +141,7 @@ static void
 set_sensitivity (E2kProperties *props, ECalComponent *comp)
 {
         ECalComponentClassification classif;
-        int sensitivity;
+        gint sensitivity;
 
 	e_cal_component_get_classification (E_CAL_COMPONENT (comp), &classif);
         switch (classif) {
@@ -163,27 +162,28 @@ set_sensitivity (E2kProperties *props, ECalComponent *comp)
 icaltimezone *
 get_default_timezone (void)
 {
-        static EConfigListener *cl = NULL;
-        gchar *location;
-        icaltimezone *local_timezone;
+	GConfClient *client;
+	icaltimezone *local_timezone;
+	const gchar *key;
+	gchar *location;
 
-	if (!cl)
-                cl = e_config_listener_new ();
+	client = gconf_client_get_default ();
+	key = "/apps/evolution/calendar/display/timezone";
+	location = gconf_client_get_string (client, key, NULL);
 
-	location = e_config_listener_get_string_with_default (cl, "/apps/evolution/calendar/display/timezone", "UTC", NULL);
-
-        if (location && location[0]) {
-                local_timezone = icaltimezone_get_builtin_timezone (location);
-        } else {
-                local_timezone = icaltimezone_get_utc_timezone ();
-        }
+	if (location != NULL && *location != '\0')
+		local_timezone = icaltimezone_get_builtin_timezone (location);
+	else
+		local_timezone = icaltimezone_get_utc_timezone ();
 
 	g_free (location);
 
+	g_object_unref (client);
+
 	return local_timezone;
 }
 
-char *
+gchar *
 calcomponentdatetime_to_string (ECalComponentDateTime *dt,
                                 icaltimezone *izone)
 {
@@ -200,7 +200,7 @@ calcomponentdatetime_to_string (ECalComponentDateTime *dt,
 	return e2k_make_timestamp (tt);
 }
 
-static char *
+static gchar *
 convert_to_utc (ECalComponentDateTime *dt)
 {
         icaltimezone *from_zone;
@@ -216,12 +216,11 @@ convert_to_utc (ECalComponentDateTime *dt)
 	return calcomponentdatetime_to_string (dt, utc_zone);
 }
 
-
 static void
 set_dtstart (E2kProperties *props, ECalComponent *comp)
 {
         ECalComponentDateTime dt;
-        char *dtstart_str;
+        gchar *dtstart_str;
 
 	e_cal_component_get_dtstart (E_CAL_COMPONENT (comp), &dt);
         if (!dt.value || icaltime_is_null_time (*dt.value)) {
@@ -239,7 +238,7 @@ static void
 set_due_date (E2kProperties *props, ECalComponent *comp)
 {
         ECalComponentDateTime dt;
-        char *due_str;
+        gchar *due_str;
 
 	e_cal_component_get_due (E_CAL_COMPONENT (comp), &dt);
         if (!dt.value || icaltime_is_null_time (*dt.value)) {
@@ -253,7 +252,7 @@ set_due_date (E2kProperties *props, ECalComponent *comp)
         e2k_properties_set_date (props, E2K_PR_MAPI_COMMON_END, due_str);
 }
 
-char *
+gchar *
 icaltime_to_e2k_time (struct icaltimetype *itt)
 {
         time_t tt;
@@ -268,7 +267,7 @@ static void
 set_date_completed (E2kProperties *props, ECalComponent *comp)
 {
         struct icaltimetype *itt;
-        char *tstr;
+        gchar *tstr;
 
 	e_cal_component_get_completed (E_CAL_COMPONENT (comp), &itt);
         if (!itt || icaltime_is_null_time (*itt)) {
@@ -277,7 +276,7 @@ set_date_completed (E2kProperties *props, ECalComponent *comp)
         }
 
 	icaltimezone_convert_time (itt,
-				   icaltimezone_get_builtin_timezone ((const char *)itt->zone),
+				   icaltimezone_get_builtin_timezone ((const gchar *)itt->zone),
 				   icaltimezone_get_utc_timezone ());
         tstr = icaltime_to_e2k_time (itt);
         e_cal_component_free_icaltimetype (itt);
@@ -289,7 +288,7 @@ static void
 set_status (E2kProperties *props, ECalComponent *comp)
 {
         icalproperty_status ical_status;
-        int status;
+        gint status;
 
 	e_cal_component_get_status (E_CAL_COMPONENT (comp), &ical_status);
         switch (ical_status) {
@@ -321,12 +320,12 @@ set_status (E2kProperties *props, ECalComponent *comp)
 static void
 set_percent (E2kProperties *props, ECalComponent *comp)
 {
-        int *percent;
-        float res;
+        gint *percent;
+        gfloat res;
 
 	e_cal_component_get_percent (E_CAL_COMPONENT (comp), &percent);
         if (percent) {
-                res = (float) *percent / 100.0;
+                res = (gfloat) *percent / 100.0;
                 e_cal_component_free_percent (percent);
         } else
                 res = 0.;
@@ -349,7 +348,7 @@ set_categories (E2kProperties *props, ECalComponent *comp)
 
 	array = g_ptr_array_new ();
         for (sl = categories; sl != NULL; sl = sl->next) {
-                char *cat = (char *) sl->data;
+                gchar *cat = (gchar *) sl->data;
 
 		if (cat)
                         g_ptr_array_add (array, g_strdup (cat));
@@ -362,7 +361,7 @@ set_categories (E2kProperties *props, ECalComponent *comp)
 static void
 set_url (E2kProperties *props, ECalComponent *comp)
 {
-        const char *url;
+        const gchar *url;
 
 	e_cal_component_get_url (E_CAL_COMPONENT (comp), &url);
         if (url)
@@ -392,11 +391,11 @@ update_props (ECalComponent *comp, E2kProperties **properties)
 	set_url (props, E_CAL_COMPONENT (comp));
 }
 
-static const char *
+static const gchar *
 get_priority (ECalComponent *comp)
 {
-	int *priority;
-	const char *result;
+	gint *priority;
+	const gchar *result;
 
 	e_cal_component_get_priority (E_CAL_COMPONENT (comp), &priority);
 
@@ -417,16 +416,16 @@ get_priority (ECalComponent *comp)
 	return result;
 }
 
-static const char *
+static const gchar *
 get_uid (ECalComponent *comp)
 {
-	const char *uid;
+	const gchar *uid;
 
 	e_cal_component_get_uid (E_CAL_COMPONENT(comp), &uid);
 	return uid;
 }
 
-static const char *
+static const gchar *
 get_summary (ECalComponent *comp)
 {
 	ECalComponentText summary;
@@ -436,18 +435,18 @@ get_summary (ECalComponent *comp)
 	return summary.value;
 }
 
-static int
+static gint
 put_body (ECalComponent *comp, E2kContext *ctx, E2kOperation *op,
-         const char *uri, const char *from_name, const char *from_addr,
-	 const char *attach_body, const char *boundary,
-         char **repl_uid)
+         const gchar *uri, const gchar *from_name, const gchar *from_addr,
+	 const gchar *attach_body, const gchar *boundary,
+         gchar **repl_uid)
 
 {
         GSList *desc_list;
         GString *desc;
-        char *desc_crlf;
-        char *body, *date;
-        int status;
+        gchar *desc_crlf;
+        gchar *body, *date;
+        gint status;
 
         /* get the description */
         e_cal_component_get_description_list (E_CAL_COMPONENT (comp), &desc_list);
@@ -464,7 +463,7 @@ put_body (ECalComponent *comp, E2kContext *ctx, E2kOperation *op,
         }
 
 	/* PUT the component on the server */
-        desc_crlf = e2k_lf_to_crlf ((const char *) desc->str);
+        desc_crlf = e2k_lf_to_crlf ((const gchar *) desc->str);
         date = e2k_make_timestamp_rfc822 (time (NULL));
 
 	if (attach_body) {
@@ -537,7 +536,7 @@ put_body (ECalComponent *comp, E2kContext *ctx, E2kOperation *op,
         return status;
 }
 
-static const char *task_props[] = {
+static const gchar *task_props[] = {
         E2K_PR_EXCHANGE_MESSAGE_CLASS,
         E2K_PR_DAV_UID,
         E2K_PR_CALENDAR_UID,
@@ -559,7 +558,6 @@ static const char *task_props[] = {
         E2K_PR_EXCHANGE_KEYWORDS,
         E2K_PR_CALENDAR_URL
 };
-static const int n_task_props = sizeof (task_props) / sizeof (task_props[0]);
 
 static guint
 get_changed_tasks (ECalBackendExchange *cbex)
@@ -572,17 +570,17 @@ get_changed_tasks (ECalBackendExchange *cbex)
 	GSList *attachment_list = NULL;
 	E2kResult *result;
 	E2kContext *ctx;
-	const char *modtime, *str, *prop;
-	char *uid;
-	const char *tzid;
-	int status, i, priority, percent;
-	float f_percent;
+	const gchar *modtime, *str, *prop;
+	gchar *uid;
+	const gchar *tzid;
+	gint status, i, priority, percent;
+	gfloat f_percent;
 	ECalComponent *ecal, *ecomp;
 	struct icaltimetype itt;
 	const icaltimezone *itzone;
 	ECalComponentDateTime ecdatetime;
 	icalcomponent *icalcomp;
-	const char *since = NULL;
+	const gchar *since = NULL;
 	ECalBackendExchangeTasks *cbext = E_CAL_BACKEND_EXCHANGE_TASKS (cbex);
 
         g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE (cbex), SOUP_STATUS_CANCELLED);
@@ -612,7 +610,7 @@ get_changed_tasks (ECalBackendExchange *cbex)
 
         iter = e_folder_exchange_search_start (cbex->folder, NULL,
 					       task_props,
-					       n_task_props,
+					       G_N_ELEMENTS (task_props),
 					       rn, NULL, TRUE);
         e2k_restriction_unref (rn);
 
@@ -635,7 +633,7 @@ get_changed_tasks (ECalBackendExchange *cbex)
 		ecal = e_cal_component_new ();
 		icalcomp = icalcomponent_new_vtodo ();
 		e_cal_component_set_icalcomponent (ecal, icalcomp);
-		e_cal_component_set_uid (ecal, (const char *)uid);
+		e_cal_component_set_uid (ecal, (const gchar *)uid);
 
 		modtime = e2k_properties_get_prop (result->props,
 						   E2K_PR_DAV_LAST_MODIFIED);
@@ -648,7 +646,7 @@ get_changed_tasks (ECalBackendExchange *cbex)
 		}
 		e_cal_backend_exchange_cache_unlock (cbex);
 
-		e_cal_backend_exchange_add_timezone (cbex, icalcomp);
+		e_cal_backend_exchange_add_timezone (cbex, icalcomp, NULL);
 
 		itt = icaltime_from_timet (e2k_parse_timestamp (modtime), 0);
 		if (!icaltime_is_null_time (itt)) {
@@ -667,7 +665,7 @@ get_changed_tasks (ECalBackendExchange *cbex)
 				priority = 5;
 			else
 				priority = 0;
-		} else 
+		} else
 			priority = 5;
 		e_cal_component_set_priority (ecal, &priority);
 
@@ -704,8 +702,8 @@ get_changed_tasks (ECalBackendExchange *cbex)
 			text.altrep = result->href;
 			sl.data = &text;
 			sl.next = NULL;
-                	e_cal_component_set_description_list (E_CAL_COMPONENT (ecal), &sl);
-			g_free ((char *)text.value);
+			e_cal_component_set_description_list (E_CAL_COMPONENT (ecal), &sl);
+			g_free ((gchar *)text.value);
 		}
 
 		/* Set DUE */
@@ -735,7 +733,7 @@ get_changed_tasks (ECalBackendExchange *cbex)
 
 		/* Set CLASSIFICATION */
 		if ((str = e2k_properties_get_prop (result->props,
-				E2K_PR_MAPI_SENSITIVITY))){
+				E2K_PR_MAPI_SENSITIVITY))) {
 			if (!strcmp (str, "0"))
 				e_cal_component_set_classification (ecal,
 					E_CAL_COMPONENT_CLASS_PUBLIC);
@@ -752,7 +750,7 @@ get_changed_tasks (ECalBackendExchange *cbex)
 				E2K_PR_OUTLOOK_TASK_PERCENT))) {
 
 			f_percent = atof (str);
-			percent = (int) (f_percent * 100);
+			percent = (gint) (f_percent * 100);
 			e_cal_component_set_percent (ecal, &percent);
 		}
 
@@ -804,7 +802,7 @@ get_changed_tasks (ECalBackendExchange *cbex)
 		if ((array = e2k_properties_get_prop (result->props,
 				E2K_PR_EXCHANGE_KEYWORDS))) {
 			GSList *list = NULL;
-			int i;
+			gint i;
 
 			for (i = 0; i < array->len; i++)
 				list = g_slist_prepend (list, array->pdata[i]);
@@ -837,12 +835,14 @@ get_changed_tasks (ECalBackendExchange *cbex)
 			e_cal_backend_exchange_cache_unlock (cbex);
 
 			if (status && kind == ICAL_VTODO_COMPONENT) {
-				char *str = icalcomponent_as_ical_string_r (icalcomp);
+				gchar *str = icalcomponent_as_ical_string_r (icalcomp);
 				e_cal_backend_notify_object_created (E_CAL_BACKEND (cbex), str);
 				g_free (str);
 			}
 
 		}
+
+		g_object_unref (ecal);
 	} /* End while */
 	status = e2k_result_iter_free (iter);
 
@@ -873,7 +873,7 @@ get_changed_tasks (ECalBackendExchange *cbex)
 
 	prop = PR_INTERNET_CONTENT;
 	iter = e_folder_exchange_bpropfind_start (cbex->folder, NULL,
-						(const char **)hrefs->pdata,
+						(const gchar **)hrefs->pdata,
 						hrefs->len, &prop, 1);
 	for (i = 0; i < hrefs->len; i++)
 		g_free (hrefs->pdata[i]);
@@ -894,7 +894,7 @@ get_changed_tasks (ECalBackendExchange *cbex)
 
 		e_cal_backend_exchange_cache_lock (cbex);
 		ecalbexcomp = get_exchange_comp (cbex, uid);
-		attachment_list = get_attachment (cbex, uid, (char *) ical_data->data, ical_data->len);
+		attachment_list = get_attachment (cbex, uid, (gchar *) ical_data->data, ical_data->len);
 		if (attachment_list) {
 			ecomp = e_cal_component_new ();
 			e_cal_component_set_icalcomponent (ecomp, icalcomponent_new_clone (ecalbexcomp->icomp));
@@ -972,7 +972,7 @@ get_changed_tasks (ECalBackendExchange *cbex)
 
 /* folder subscription notify callback */
 static void
-notify_changes (E2kContext *ctx, const char *uri,
+notify_changes (E2kContext *ctx, const gchar *uri,
                      E2kContextChangeType type, gpointer user_data)
 {
 
@@ -985,28 +985,29 @@ notify_changes (E2kContext *ctx, const char *uri,
 
 }
 
-static ECalBackendSyncStatus
+static void
 open_task (ECalBackendSync *backend, EDataCal *cal,
 	   gboolean only_if_exits,
-	   const char *username, const char *password)
+	   const gchar *username, const gchar *password, GError **perror)
 {
-	ECalBackendSyncStatus status;
 	GThread *thread = NULL;
 	GError *error = NULL;
 	ECalBackendExchangeTasks *cbext = E_CAL_BACKEND_EXCHANGE_TASKS (backend);
 
-	status = E_CAL_BACKEND_SYNC_CLASS (parent_class)->open_sync (backend,
-				     cal, only_if_exits, username, password);
-	if (status != GNOME_Evolution_Calendar_Success)
-		return status;
+	E_CAL_BACKEND_SYNC_CLASS (parent_class)->open_sync (backend,
+				     cal, only_if_exits, username, password, &error);
+	if (error) {
+		g_propagate_error (perror, error);
+		return;
+	}
 
 	if (!e_cal_backend_exchange_is_online (E_CAL_BACKEND_EXCHANGE (backend))) {
 		d(printf ("ECBEC : calendar is offline\n"));
-		return GNOME_Evolution_Calendar_Success;
+		return;
 	}
 
 	if (cbext->priv->is_loaded)
-		return GNOME_Evolution_Calendar_Success;
+		return;
 
 	/* Subscribe to the folder to notice changes */
         e_folder_exchange_subscribe (E_CAL_BACKEND_EXCHANGE (backend)->folder,
@@ -1015,14 +1016,9 @@ open_task (ECalBackendSync *backend, EDataCal *cal,
 
 	thread = g_thread_create ((GThreadFunc) get_changed_tasks, E_CAL_BACKEND_EXCHANGE (backend), FALSE, &error);
 	if (!thread) {
-		g_warning (G_STRLOC ": %s", error->message);
+		g_propagate_error (perror, EDC_ERROR_EX (OtherError, error->message));
 		g_error_free (error);
-
-		return GNOME_Evolution_Calendar_OtherError;
 	}
-
-
-	return GNOME_Evolution_Calendar_Success;
 }
 
 struct _cb_data {
@@ -1031,49 +1027,53 @@ struct _cb_data {
         EDataCal *cal;
 };
 
-static ECalBackendSyncStatus
+static void
 create_task_object (ECalBackendSync *backend, EDataCal *cal,
-		    char **calobj, char **return_uid)
+		    gchar **calobj, gchar **return_uid, GError **error)
 {
 	ECalBackendExchangeTasks *ecalbextask;
 	ECalBackendExchange *ecalbex;
 	E2kProperties *props;
 	E2kContext *e2kctx;
+	E2kHTTPStatus status;
 	ECalComponent *comp;
 	icalcomponent *icalcomp, *real_icalcomp;
 	icalcomponent_kind kind;
 	icalproperty *icalprop;
 	struct icaltimetype current;
-	char *from_name = NULL, *from_addr = NULL;
-	char *boundary = NULL;
-	char *attach_body = NULL;
-	char *attach_body_crlf = NULL;
-	const char *summary;
-	char * modtime;
-	char *location;
-	ECalBackendSyncStatus status;
-	const char *temp_comp_uid;
+	gchar *from_name = NULL, *from_addr = NULL;
+	gchar *boundary = NULL;
+	gchar *attach_body = NULL;
+	gchar *attach_body_crlf = NULL;
+	const gchar *summary;
+	gchar * modtime;
+	gchar *location;
+	const gchar *temp_comp_uid;
 
 	ecalbextask = E_CAL_BACKEND_EXCHANGE_TASKS (backend);
 	ecalbex = E_CAL_BACKEND_EXCHANGE (backend);
 
-	g_return_val_if_fail (calobj != NULL, GNOME_Evolution_Calendar_ObjectNotFound);
+	e_return_data_cal_error_if_fail (calobj != NULL, InvalidArg);
 
 	if (!e_cal_backend_exchange_is_online (E_CAL_BACKEND_EXCHANGE (backend))) {
 		d(printf ("tasks are offline\n"));
-		return GNOME_Evolution_Calendar_InvalidObject;
+		g_propagate_error (error, EDC_ERROR (RepositoryOffline));
+		return;
 	}
 
 	/* Parse the icalendar text */
 	icalcomp = icalparser_parse_string (*calobj);
-	if (!icalcomp)
-		return GNOME_Evolution_Calendar_InvalidObject;
+	if (!icalcomp) {
+		g_propagate_error (error, EDC_ERROR (InvalidObject));
+		return;
+	}
 
 	/* Check kind with the parent */
 	kind = e_cal_backend_get_kind (E_CAL_BACKEND (ecalbex));
         if (icalcomponent_isa (icalcomp) != kind) {
 		icalcomponent_free (icalcomp);
-		return GNOME_Evolution_Calendar_InvalidObject;
+		g_propagate_error (error, EDC_ERROR (InvalidObject));
+		return;
         }
 
 	current = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
@@ -1095,7 +1095,8 @@ create_task_object (ECalBackendSync *backend, EDataCal *cal,
 	temp_comp_uid = icalcomponent_get_uid (icalcomp);
 	if (!temp_comp_uid) {
 		icalcomponent_free (icalcomp);
-		return GNOME_Evolution_Calendar_InvalidObject;
+		g_propagate_error (error, EDC_ERROR (InvalidObject));
+		return;
 	}
 
 	e_cal_backend_exchange_cache_lock (ecalbex);
@@ -1104,7 +1105,8 @@ create_task_object (ECalBackendSync *backend, EDataCal *cal,
 					     temp_comp_uid, modtime, NULL, NULL)) {
 		e_cal_backend_exchange_cache_unlock (ecalbex);
 		icalcomponent_free (icalcomp);
-		return GNOME_Evolution_Calendar_ObjectIdAlreadyExists;
+		g_propagate_error (error, EDC_ERROR (ObjectIdAlreadyExists));
+		return;
 	}
 	e_cal_backend_exchange_cache_unlock (ecalbex);
 
@@ -1159,11 +1161,12 @@ create_task_object (ECalBackendSync *backend, EDataCal *cal,
 	update_props (comp, &props);
 	e_cal_component_commit_sequence (comp);
 	*calobj = e_cal_component_get_as_string (comp);
-	if (!*calobj){
+	if (!*calobj) {
 		g_object_unref (comp);
 		g_free (from_name);
 		g_free (from_addr);
-		return GNOME_Evolution_Calendar_OtherError;
+		g_propagate_error (error, EDC_ERROR_EX (OtherError, "Cannot get ECalComp as string"));
+		return;
 	}
 
 	real_icalcomp = icalparser_parse_string (*calobj);
@@ -1186,17 +1189,19 @@ create_task_object (ECalBackendSync *backend, EDataCal *cal,
 		g_free (modtime);
 	}
 
+	if (!E2K_HTTP_STATUS_IS_SUCCESSFUL (status))
+		g_propagate_error (error, EDC_ERROR_HTTP_STATUS (status));
+
 	*return_uid = g_strdup (temp_comp_uid);
 	icalcomponent_free (real_icalcomp);
 	g_free (from_name);
 	g_free (from_addr);
-	return GNOME_Evolution_Calendar_Success;
 }
 
-static ECalBackendSyncStatus
+static void
 modify_task_object (ECalBackendSync *backend, EDataCal *cal,
-	       const char *calobj, CalObjModType mod,
-	       char **old_object, char **new_object)
+	       const gchar *calobj, CalObjModType mod,
+	       gchar **old_object, gchar **new_object, GError **error)
 {
 	ECalBackendExchangeTasks *ecalbextask;
 	ECalBackendExchangeComponent *ecalbexcomp;
@@ -1204,39 +1209,41 @@ modify_task_object (ECalBackendSync *backend, EDataCal *cal,
 	ECalBackendExchange *ecalbex;
 	E2kProperties *props;
 	icalcomponent *icalcomp;
-	const char* comp_uid, *summary;
-	char *from_name, *from_addr;
-	char *comp_str;
-	char *attach_body = NULL;
-	char *attach_body_crlf = NULL;
-	char *boundary = NULL;
+	const gchar * comp_uid, *summary;
+	gchar *from_name, *from_addr;
+	gchar *comp_str;
+	gchar *attach_body = NULL;
+	gchar *attach_body_crlf = NULL;
+	gchar *boundary = NULL;
 	struct icaltimetype current;
-	ECalBackendSyncStatus status;
 	E2kContext *e2kctx;
+	E2kHTTPStatus status;
 
 	ecalbextask = E_CAL_BACKEND_EXCHANGE_TASKS (backend);
 	ecalbex = E_CAL_BACKEND_EXCHANGE (backend);
 
-	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE_TASKS (ecalbextask),
-					GNOME_Evolution_Calendar_NoSuchCal);
-	g_return_val_if_fail (calobj != NULL,
-				GNOME_Evolution_Calendar_ObjectNotFound);
+	e_return_data_cal_error_if_fail (E_IS_CAL_BACKEND_EXCHANGE_TASKS (ecalbextask), InvalidArg);
+	e_return_data_cal_error_if_fail (calobj != NULL, InvalidArg);
 
 	if (!e_cal_backend_exchange_is_online (E_CAL_BACKEND_EXCHANGE (backend))) {
 		d(printf ("tasks are offline\n"));
-		return GNOME_Evolution_Calendar_InvalidObject;
+		g_propagate_error (error, EDC_ERROR (RepositoryOffline));
+		return;
 	}
 
 	/* Parse the icalendar text */
-        icalcomp = icalparser_parse_string ((char *) calobj);
-        if (!icalcomp)
-                return GNOME_Evolution_Calendar_InvalidObject;
+        icalcomp = icalparser_parse_string ((gchar *) calobj);
+        if (!icalcomp) {
+                g_propagate_error (error, EDC_ERROR (InvalidObject));
+		return;
+	}
 
 	/* Check kind with the parent */
         if (icalcomponent_isa (icalcomp) !=
 			e_cal_backend_get_kind (E_CAL_BACKEND (backend))) {
                 icalcomponent_free (icalcomp);
-                return GNOME_Evolution_Calendar_InvalidObject;
+		g_propagate_error (error, EDC_ERROR (InvalidObject));
+		return;
         }
 
 	/* Get the uid */
@@ -1250,7 +1257,8 @@ modify_task_object (ECalBackendSync *backend, EDataCal *cal,
 	if (!ecalbexcomp) {
 		icalcomponent_free (icalcomp);
 		e_cal_backend_exchange_cache_unlock (ecalbex);
-		return GNOME_Evolution_Calendar_ObjectNotFound;
+		g_propagate_error (error, EDC_ERROR (ObjectNotFound));
+		return;
 	}
 
         cache_comp = e_cal_component_new ();
@@ -1285,8 +1293,10 @@ modify_task_object (ECalBackendSync *backend, EDataCal *cal,
 	comp_str = e_cal_component_get_as_string (new_comp);
 	icalcomp = icalparser_parse_string (comp_str);
 	g_free (comp_str);
-	if (!icalcomp)
-		return GNOME_Evolution_Calendar_OtherError;
+	if (!icalcomp) {
+		g_propagate_error (error, EDC_ERROR_EX (OtherError, "Failed to parse comp_str"));
+		return;
+	}
 	icalcomponent_free (icalcomp);
 
 	get_from (backend, new_comp, &from_name, &from_addr);
@@ -1301,7 +1311,7 @@ modify_task_object (ECalBackendSync *backend, EDataCal *cal,
 	comp_str = e_cal_component_get_as_string (new_comp);
 	icalcomp = icalparser_parse_string (comp_str);
 	g_free (comp_str);
-	if (E2K_HTTP_STATUS_IS_SUCCESSFUL (status)){
+	if (E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
 		status = put_body(new_comp, e2kctx, NULL, ecalbexcomp->href, from_name, from_addr,
 					attach_body_crlf, boundary, NULL);
 		if (E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
@@ -1311,12 +1321,14 @@ modify_task_object (ECalBackendSync *backend, EDataCal *cal,
 		}
 	}
 	icalcomponent_free (icalcomp);
-	return GNOME_Evolution_Calendar_Success;
+
+	if (!E2K_HTTP_STATUS_IS_SUCCESSFUL (status))
+		g_propagate_error (error, EDC_ERROR_HTTP_STATUS (status));
 }
 
-static ECalBackendSyncStatus
+static void
 receive_task_objects (ECalBackendSync *backend, EDataCal *cal,
-                 const char *calobj)
+                 const gchar *calobj, GError **error)
 {
 	ECalBackendExchangeTasks *ecalbextask;
 	ECalBackendExchange *cbex;
@@ -1325,28 +1337,26 @@ receive_task_objects (ECalBackendSync *backend, EDataCal *cal,
         struct icaltimetype current;
         icalproperty_method method;
         icalcomponent *subcomp;
-        ECalBackendSyncStatus status = GNOME_Evolution_Calendar_Success;
+	GError *err = NULL;
 
 	ecalbextask = E_CAL_BACKEND_EXCHANGE_TASKS (backend);
 	cbex = E_CAL_BACKEND_EXCHANGE (backend);
 
-	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE_TASKS (ecalbextask),
-                                        GNOME_Evolution_Calendar_NoSuchCal);
-        g_return_val_if_fail (calobj != NULL,
-                                GNOME_Evolution_Calendar_ObjectNotFound);
+	e_return_data_cal_error_if_fail (E_IS_CAL_BACKEND_EXCHANGE_TASKS (ecalbextask), InvalidArg);
+        e_return_data_cal_error_if_fail (calobj != NULL, InvalidArg);
 
 	if (!e_cal_backend_exchange_is_online (E_CAL_BACKEND_EXCHANGE (backend))) {
 		d(printf ("tasks are offline\n"));
-		return GNOME_Evolution_Calendar_InvalidObject;
+		g_propagate_error (error, EDC_ERROR (RepositoryOffline));
+		return;
 	}
 
-	status = e_cal_backend_exchange_extract_components (calobj, &method, &comps);
-        if (status != GNOME_Evolution_Calendar_Success)
-                return GNOME_Evolution_Calendar_InvalidObject;
+	if (!e_cal_backend_exchange_extract_components (calobj, &method, &comps, error))
+		return;
 
 	for (l = comps; l; l = l->next) {
-                const char *uid;
-                char *calobj, *rid = NULL;
+                const gchar *uid;
+                gchar *calobj, *rid = NULL;
 
                 subcomp = l->data;
 
@@ -1366,27 +1376,29 @@ receive_task_objects (ECalBackendSync *backend, EDataCal *cal,
 
 		e_cal_backend_exchange_cache_lock (cbex);
                 if (get_exchange_comp (E_CAL_BACKEND_EXCHANGE (ecalbextask), uid)) {
-                        char *old_object;
+                        gchar *old_object;
 
 			e_cal_backend_exchange_cache_unlock (cbex);
-                        status = modify_task_object (backend, cal, calobj, CALOBJ_MOD_THIS, &old_object, NULL);
-                        if (status != GNOME_Evolution_Calendar_Success) {
+                        modify_task_object (backend, cal, calobj, CALOBJ_MOD_THIS, &old_object, NULL, &err);
+                        if (err) {
 				g_free (rid);
-                                goto error;
+				g_propagate_error (error, err);
+                                return;
 			}
 
                         e_cal_backend_notify_object_modified (E_CAL_BACKEND (backend), old_object, calobj);
 			g_free (old_object);
                 } else {
-                        char *returned_uid;
+                        gchar *returned_uid;
 
 			e_cal_backend_exchange_cache_unlock (cbex);
-			calobj = (char *) icalcomponent_as_ical_string_r (subcomp);
-			status = create_task_object (backend, cal, &calobj, &returned_uid);
-                        if (status != GNOME_Evolution_Calendar_Success) {
+			calobj = (gchar *) icalcomponent_as_ical_string_r (subcomp);
+			create_task_object (backend, cal, &calobj, &returned_uid, &err);
+                        if (err) {
 				g_free (calobj);
 				g_free (rid);
-                                goto error;
+				g_propagate_error (error, err);
+                                return;
 			}
 
                         e_cal_backend_notify_object_created (E_CAL_BACKEND (backend), calobj);
@@ -1396,27 +1408,25 @@ receive_task_objects (ECalBackendSync *backend, EDataCal *cal,
         }
 
         g_list_free (comps);
-error:
-        return status;
 }
 
-static ECalBackendSyncStatus
+static void
 remove_task_object (ECalBackendSync *backend, EDataCal *cal,
-	       const char *uid, const char *rid, CalObjModType mod,
-	       char **old_object, char **object)
+	       const gchar *uid, const gchar *rid, CalObjModType mod,
+	       gchar **old_object, gchar **object, GError **error)
 {
 	ECalBackendExchange *ecalbex = E_CAL_BACKEND_EXCHANGE (backend);
 	ECalBackendExchangeComponent *ecalbexcomp;
 	ECalComponent *comp;
 	E2kContext *ctx;
-	int status;
+	E2kHTTPStatus status;
 
-	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE (ecalbex),
-					GNOME_Evolution_Calendar_OtherError);
+	e_return_data_cal_error_if_fail (E_IS_CAL_BACKEND_EXCHANGE (ecalbex), InvalidArg);
 
 	if (!e_cal_backend_exchange_is_online (E_CAL_BACKEND_EXCHANGE (backend))) {
 		d(printf ("tasks are offline\n"));
-		return GNOME_Evolution_Calendar_InvalidObject;
+		g_propagate_error (error, EDC_ERROR (RepositoryOffline));
+		return;
 	}
 
 	e_cal_backend_exchange_cache_lock (ecalbex);
@@ -1424,7 +1434,8 @@ remove_task_object (ECalBackendSync *backend, EDataCal *cal,
 
 	if (!ecalbexcomp || !ecalbexcomp->href) {
 		e_cal_backend_exchange_cache_unlock (ecalbex);
-		return GNOME_Evolution_Calendar_ObjectNotFound;
+		g_propagate_error (error, EDC_ERROR (ObjectNotFound));
+		return;
 	}
 
         comp = e_cal_component_new ();
@@ -1439,9 +1450,10 @@ remove_task_object (ECalBackendSync *backend, EDataCal *cal,
 	status = e2k_context_delete (ctx, NULL, ecalbexcomp->href);
 	if (E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
 		if (e_cal_backend_exchange_remove_object (ecalbex, uid))
-			return GNOME_Evolution_Calendar_Success;
+			return;
 	}
-	return GNOME_Evolution_Calendar_OtherError;
+
+	g_propagate_error (error, EDC_ERROR_HTTP_STATUS (status));
 }
 
 static void
